/*
 * JBoss, Home of Professional Open Source
 * Copyright 2011, Red Hat Middleware LLC, and others contributors as indicated
 * by the @authors tag. All rights reserved.
 * See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU Lesser General Public License, v. 2.1.
 * This program is distributed in the hope that it will be useful, but WITHOUT A
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License,
 * v.2.1 along with this distribution; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 */
package org.jboss.jbw2011.keynote.demo.drools;

import java.util.ArrayList ;
import java.util.List ;

import org.jboss.jbw2011.keynote.demo.drools.model.Tweet ;
import org.junit.Test ;
import static org.junit.Assert.assertEquals ;
import static org.junit.Assert.assertNotNull ;


public class RulesServiceUnitTest
{
    @Test
    public void testAllVotes()
        throws RulesServiceException
    {
        final TestTweetProcessor testTweetProcessor = new TestTweetProcessor() ;
        
        final RulesService rulesService = new RulesService("allVotes.drl", testTweetProcessor) ;
        try
        {
            addTweets(rulesService) ;
            
            final List<Tweet> processedTweets = testTweetProcessor.getProcessedTweets() ;
            checkValid(processedTweets, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11) ;
        }
        finally
        {
            rulesService.dispose() ;
        }
    }

    @Test
    public void testFilteredVotes()
        throws RulesServiceException
    {
        final TestTweetProcessor testTweetProcessor = new TestTweetProcessor() ;
        
        final RulesService rulesService = new RulesService("filterVotes.drl", testTweetProcessor) ;
        
        try
        {
            addTweets(rulesService) ;
            
            final List<Tweet> processedTweets = testTweetProcessor.getProcessedTweets() ;
            checkValid(processedTweets, 1, 7, 8, 9, 10, 11) ;
        }
        finally
        {
            rulesService.dispose() ;
        }
    }

    private static void addTweets(final RulesService rulesService)
        throws RulesServiceException
    {
        // During filter, only the first and seventh will be passed through
        rulesService.processTweet(new Tweet(1, "message", "byteman", "screenName1", 1000L)) ;
        rulesService.processTweet(new Tweet(2, "message", "byteman", "screenName1", 1000L)) ;
        rulesService.processTweet(new Tweet(3, "message", "byteman", "screenName1", 61000L)) ;
        rulesService.processTweet(new Tweet(4, "message", "byteman", "screenName1", 121000L)) ;
        rulesService.processTweet(new Tweet(5, "message", "byteman", "screenName1", 179999L)) ;
        rulesService.processTweet(new Tweet(6, "message", "byteman", "screenName1", 180000L)) ;
        rulesService.processTweet(new Tweet(7, "message", "byteman", "screenName1", 181000L)) ;

        // all valid votes
        rulesService.processTweet(new Tweet(8, "message", "drools", "screenName1", 361000L)) ;
        rulesService.processTweet(new Tweet(9, "message", "switchyard", "screenName1", 361000L)) ;
        // all alid votes, second person
        rulesService.processTweet(new Tweet(10, "message", "drools", "screenName2", 361000L)) ;
        rulesService.processTweet(new Tweet(11, "message", "switchyard", "screenName2", 361000L)) ;
    }

    private static void checkValid(final List<Tweet> processedTweets, int ... ids)
    {
        assertNotNull("Invalid ids", ids) ;
        final int length = ids.length ;
        assertEquals("Invalid process tweet length", length, processedTweets.size()) ;
        
        for(int count = 0 ; count < length ; count++)
        {
            assertEquals("Invalid tweet: " + count, ids[count], processedTweets.get(count).getId()) ;
        }
    }

    private static class TestTweetProcessor implements TweetProcessor
    {
        private List<Tweet> processedTweets = new ArrayList<Tweet>() ;
        
        @Override
        public void processTweet(final Tweet tweet)
        {
            processedTweets.add(tweet) ;
        }
        
        public List<Tweet> getProcessedTweets()
        {
            return processedTweets ;
        }
    }
}
